/*
 * @(#)file      SnmpJdmkRfc2573NotificationOriginator.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.14
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573;

import java.io.File;
import java.util.Vector;
import java.util.Enumeration;
import java.io.IOException;
import java.net.InetAddress;

import com.sun.management.comm.SnmpV3AdaptorServer;
import com.sun.management.snmp.agent.SnmpMibRequest;
import com.sun.management.snmp.agent.SnmpMibAgent;
import com.sun.management.snmp.agent.SnmpRequestForwarder;
import com.sun.management.snmp.agent.SnmpProxy;
import com.sun.management.internal.snmp.SnmpTools;

import com.sun.jdmk.trace.Trace;

import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.SnmpScopedPduPacket;
import com.sun.management.snmp.SnmpScopedPduRequest;
import com.sun.management.snmp.SnmpPduBulkType;
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpAckPdu;
import com.sun.management.snmp.SnmpPdu;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpEngineId;
import com.sun.management.snmp.SnmpEngine;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpIpAddress;
import com.sun.management.snmp.SnmpTimeticks;
import com.sun.management.snmp.manager.usm.SnmpUsmPeer;
import com.sun.management.snmp.UserAcl;
import com.sun.management.snmp.InetAddressAcl;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.MalformedObjectNameException;
import com.sun.management.snmp.SnmpEngineFactory;
import com.sun.management.snmp.SnmpEngineParameters;
import com.sun.management.snmp.manager.usm.SnmpUsmParameters;

import com.sun.jdmk.ServiceName;
import com.sun.management.comm.SnmpInformHandler;
import com.sun.management.comm.SnmpInformRequest;
import com.sun.management.internal.snmp.SnmpEngineImpl;
import com.sun.management.internal.snmp.SnmpEngineImpl;
import com.sun.management.snmp.JdmkEngineFactory;
import com.sun.management.snmp.agent.SnmpTrap;

import com.sun.management.snmp.rfc2573.target.mib.SNMP_TARGET_MIB;
import com.sun.management.snmp.rfc2573.target.mib.SNMP_TARGET_MIBImpl;
import com.sun.management.snmp.rfc2573.internal.target.SnmpJdmkTargetLcd;
import com.sun.management.snmp.rfc2573.target.SnmpTargetLcd;
import com.sun.management.snmp.rfc2573.target.SnmpTargetData;

import com.sun.management.snmp.rfc2573.notification.mib.SNMP_NOTIFICATION_MIB;
import com.sun.management.snmp.rfc2573.notification.mib.SNMP_NOTIFICATION_MIBImpl;
import com.sun.management.snmp.rfc2573.internal.notification.SnmpJdmkNotificationLcd;
import com.sun.management.snmp.rfc2573.notification.SnmpNotificationLcd;
import com.sun.management.snmp.rfc2573.notification.SnmpNotificationData;

public class SnmpJdmkRfc2573NotificationOriginator {

    SnmpV3AdaptorServer server = null;    
    MBeanServer mbserver = null;
    SnmpNotificationLcd notifLcd = null;
    SnmpTargetLcd targetLcd = null;
    String notifsFile = null;
    String targetsFile = null;
    String paramsFile=null;
    SNMP_TARGET_MIB targetMib = null;
    SNMP_NOTIFICATION_MIB notifMib = null;


    public SnmpJdmkRfc2573NotificationOriginator(SnmpV3AdaptorServer server,
						 MBeanServer mbserver,
						 String paramsFile,
						 String targetsFile,
						 String notifsFile) {
	this.mbserver = mbserver;
	this.server = server;
	this.paramsFile = paramsFile;
	this.targetsFile = targetsFile;
	this.notifsFile = notifsFile;
    }

    public SnmpJdmkRfc2573NotificationOriginator(SnmpV3AdaptorServer server,
					 MBeanServer mbserver,
					 SnmpNotificationLcd notifLcd,
					 SnmpTargetLcd targetLcd) {
	this.mbserver = mbserver;
	this.server = server;
	this.notifLcd = notifLcd;
	this.targetLcd = targetLcd;
    }

    public SnmpJdmkRfc2573NotificationOriginator(SnmpV3AdaptorServer server,
					 MBeanServer mbserver,
					 String notifsFile,
					 SnmpTargetLcd targetLcd) {
	this.mbserver = mbserver;
	this.server = server;
	this.notifsFile = notifsFile;
	this.targetLcd = targetLcd;
    }

    /**
     * Set your our own RFC 2573 MIB target implementation.
     */

    public void setRfc2573TargetMib(SNMP_TARGET_MIB mib) {
	targetMib = mib;
    }

    /** 
     * Get RFC 2573 MIB target implementation. 
     */

    public SNMP_TARGET_MIB getRfc2573TargetMib() {
	return targetMib;
    }

    public void setRfc2573NotificationMib(SNMP_NOTIFICATION_MIB mib) {
	notifMib = mib;
    }

    /**
     * Get RFC 2573 MIB target implementation. 
     */

    public SNMP_NOTIFICATION_MIB getRfc2573NotificationMib() {
	return notifMib;
    }

    public void start() throws Exception {

	// Read target configuration
	if(targetLcd == null) 
	    targetLcd = new SnmpJdmkTargetLcd(server.getEngine(),
					      paramsFile,
					      targetsFile);

	if (isTraceOn()) {
	    debug("initConfiguration", "Read target configuration ");
	}

	targetLcd.readConfiguration();
	
	// Read notification configuration
	if(notifLcd == null) 
	    notifLcd = new SnmpJdmkNotificationLcd(server.getEngine(),
						   targetLcd,
						   notifsFile);

	if (isTraceOn()) {
	    debug("initConfiguration", "Read notification configuration");
	}

	notifLcd.readConfiguration();

	// Initialize the two mibs

	if(targetMib == null) {
	    targetMib = new SNMP_TARGET_MIBImpl(targetLcd);
	}

	if(notifMib == null) {
	    notifMib = new SNMP_NOTIFICATION_MIBImpl(notifLcd);
	}

	// Register the two mibs 
	if(mbserver != null) {
	    try {
		ObjectName notifObjName =  createNotifMibName(server,
							      notifMib);

		if(isTraceOn())
		    trace("init", "notification object name : " + notifObjName);

		ObjectName targetObjName =  createTargetMibName(server,
								targetMib);

		if(isTraceOn())
		    trace("init", "target object name : " + targetObjName);
	
		if (mbserver.isRegistered(targetObjName)==false) {
		    mbserver.registerMBean(targetMib, targetObjName);
		    if(isTraceOn())
			trace("init", "target mib registered");
		}
		else {
		    if(isTraceOn())
			trace("init", "target mib already registered");
		}
			
		if (mbserver.isRegistered(notifObjName)==false) {
		    mbserver.registerMBean(notifMib, notifObjName);
		    if(isTraceOn())
			trace("init", "notification mib registered");
		}
		else {
		    if(isTraceOn())
			trace("init", "notification mib already registered");
		}
	    } 
	    catch(Exception e) {
		if (isDebugOn()) {
		    debug("init",  e.toString());
		}
	    }
	}
	else 
	    try {

		// initialize the two mibs
		targetMib.init();
		notifMib.init();
	    }
	    catch(IllegalAccessException e) {
		if (isTraceOn()) {
		    debug("init", e.toString());
		}
	    }

	// Add the two mibs in the SNMP adaptor to be accessible
	server.addMib(targetMib);
	server.addMib(notifMib);
    }

    protected ObjectName createTargetMibName(SnmpV3AdaptorServer server,
					     SNMP_TARGET_MIB mib) {
	try {
	    return new ObjectName(ServiceName.DOMAIN +":"+ "mib=" + mib.getMibName());
	}
	catch(MalformedObjectNameException e) { return null; }
    }
    
    protected ObjectName createNotifMibName(SnmpV3AdaptorServer server,
					    SNMP_NOTIFICATION_MIB mib) {
	try {
	    return new ObjectName(ServiceName.DOMAIN +":"+ "mib=" + mib.getMibName());
	}
	catch(MalformedObjectNameException e) { return null; }
    }  

    /**
     * Send a Trap to each target registered 
     */

    public void snmpTrap(int generic, int specific, SnmpVarBindList varBindList) 
	throws IOException, SnmpStatusException {
	this.snmpTrap(generic, specific, null, varBindList);
    }

    public void snmpTrap(int generic, int specific, String cs, SnmpVarBindList varBindList) 
	throws IOException, SnmpStatusException {

	// Trap received in V1
	// trap to sent to target
	SnmpTrap trap = null;
	
	/* A V1 trap SNMP message has to be sent */
	for (Enumeration notifs = notifLcd.getNotifications(); notifs.hasMoreElements();) {
        
	    /* Get one element of the list */
	    SnmpNotificationData notifdata = (SnmpNotificationData)notifs.nextElement();
   
	    /* If the notification type is a trap */
	    if (notifdata.notifyType==1) {
    	
		for (Enumeration e = notifLcd.getTargets(notifdata.notifyTag);e.hasMoreElements() ;) {
		    
		    SnmpTargetData targetdata = (SnmpTargetData) e.nextElement();
		    /* send a trap for each target in each SNMP parameters */
		    
		    if (isDebugOn()) {
			debug("snmpTrap", "Sent Trap to " + targetdata.toString());
		    }

		    /* Build a nonAuthoritativePeer for Trap*/
		    targetdata.activateAuthoritativePeer();

		    // Build the snmp Trap to send
		    trap = new SnmpTrap(targetdata.authoritativePeer.getDestAddr(), 
					cs,
					generic,
					specific,
					varBindList);

		    trap.setDestinationPort(targetdata.authoritativePeer.getDestPort());

		    /* get version of the target peer */
		    int processingModel = targetdata.params.mpm;
		 
		    // Send for params.mpm V1 V2 V3
		    if (processingModel == SnmpDefinitions.snmpV1MsgProcessingModel)
			trap.sendV1(server);
		    else if (processingModel == SnmpDefinitions.snmpV2MsgProcessingModel)
			trap.sendV2(server);
		    else 
			trap.sendV3Usm(server, 
				       ((SnmpUsmParameters)(targetdata.authoritativePeer.getParams())).getPrincipal(),
				       ((SnmpUsmParameters)(targetdata.authoritativePeer.getParams())).getSecurityLevel(),
				       null);
		}
	    }
	}
    }

    /**
     * Forward a SNMP Trap to each registered targets
     */
    public void snmpTrap(SnmpOid trapOid, SnmpVarBindList varBindList)
	throws IOException, SnmpStatusException {
	this.snmpTrap(trapOid, varBindList, null);
    }

   /**
     * Forward a SNMP Trap to each registered targets
     */
    public void snmpTrap(SnmpOid trapOid, SnmpVarBindList varBindList, String contextName)
	throws IOException, SnmpStatusException {
	
	// Trap received in V2/V3
	// trap to sent to registered targets
	SnmpTrap trap = null;

	/* A trap SNMP message has to be sent */
	for (Enumeration notifs = notifLcd.getNotifications(); notifs.hasMoreElements();) {
	    
	    /* Get one element of the list */
	    SnmpNotificationData notifdata = (SnmpNotificationData) notifs.nextElement();
	    
	    /* If the notification type is a trap */
	    if (notifdata.notifyType==1) {
	    
		// Get the interessed targets from the MIBs
		for (Enumeration e = notifLcd.getTargets(notifdata.notifyTag);e.hasMoreElements() ;) {
		    SnmpTargetData targetdata = (SnmpTargetData) e.nextElement();
  
		    if (isDebugOn()) {
			debug("snmpTrap", "Sending to " + targetdata.toString());
		    }

		    // Activate an authoritative peer
		    targetdata.activateAuthoritativePeer();

		    // Build the snmp Trap to send to the peer addr
		    trap = new SnmpTrap(targetdata.authoritativePeer.getDestAddr(), 
					null,
					trapOid,
					varBindList);

		    // Set the Trap port to the target port
		    trap.setDestinationPort(targetdata.authoritativePeer.getDestPort());
       	   
		    /* get the asked version of the peer */
		    int processingModel = targetdata.params.mpm;

		    if (processingModel == SnmpDefinitions.snmpV1MsgProcessingModel)

			// send trap in V1
			trap.sendV1(server);

		    else if (processingModel == SnmpDefinitions.snmpV2MsgProcessingModel)

			// send trap in V2
			trap.sendV2(server);
		    else {

			// send trap in V3 with the defined usm parameters 
			// Principal, Security Level, Context name
			trap.sendV3Usm(server, 
				       ((SnmpUsmParameters)(targetdata.authoritativePeer.getParams())).getPrincipal(),
				       ((SnmpUsmParameters)(targetdata.authoritativePeer.getParams())).getSecurityLevel(),
				       contextName == null ? null : new String(contextName));
		    }
		}
	    }
	}
    }




    /**
     * Forward an snmpInform request to each registered target
     */

    public Enumeration snmpInform(SnmpInformHandler cb,
		      SnmpOid trapOid, 
		      SnmpVarBindList list) 
	throws IllegalStateException,
	       IOException,
	       SnmpStatusException {
	
	SnmpInformRequest inform = null;
	Vector informList = new Vector();

	/* A trap SNMP message has to be sent */
	for (Enumeration notifs = notifLcd.getNotifications(); notifs.hasMoreElements();) {
	    
	    /* Get one element of the list */
	    SnmpNotificationData notifdata = (SnmpNotificationData) notifs.nextElement();
	    
	    /* If the notification type is a trap */
	    if (notifdata.notifyType==2) {
	    

		for (Enumeration e = notifLcd.getTargets(notifdata.notifyTag);e.hasMoreElements() ;) {
		    SnmpTargetData targetdata = (SnmpTargetData) e.nextElement();
  
		    if (isDebugOn()) {
			debug("snmpInform", "Sending Inform to " + targetdata.toString());
		    }

		    targetdata.activateAuthoritativePeer();
       	   
		    /* get version of the peer */
		    int processingModel = targetdata.params.mpm;

		    if (processingModel == SnmpDefinitions.snmpV2MsgProcessingModel) {
			inform = server.snmpInformRequest((SnmpUsmPeer)targetdata.authoritativePeer,
					       cb,
					       trapOid,
					       list);
		    }
		    else {
			targetdata.activateNonAuthoritativePeer();
			inform = server.snmpV3UsmInformRequest((SnmpUsmPeer)targetdata.nonAuthoritativePeer,
					       cb,
					       trapOid,
					       list);
		    }
		    
		    informList.add(inform);
		}
	    }
	}
	return informList.elements();
    }

    boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    void trace(String clz, String func, String info) {
	if (isTraceOn()) {
	    Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
	}
    }

    void trace(String func, String info) {
	if (isTraceOn()) {
	    trace(dbgTag, func, info);
	}
    }
    
    boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    void debug(String clz, String func, String info) {
	if (isDebugOn()) {
	    Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
	}
    }

    void debug(String clz, String func, Throwable exception) {
	if (isDebugOn()) {
	    Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
	}
    }

    void debug(String func, String info) {
	if (isDebugOn()) {
	    debug(dbgTag, func, info);
	}
    }
    
    void debug(String func, Throwable exception) {
        if (isDebugOn()) {
	    debug(dbgTag, func, exception);
	}
    }
    
    String dbgTag = "SnmpJdmkRFC2573NotificationOriginator"; 
}
