/*
 * @(#)file      SnmpJdmkRfc2573ProxyForwarder.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.13
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573;

import java.util.Vector;
import com.sun.management.comm.SnmpV3AdaptorServer;
import com.sun.management.snmp.agent.SnmpMibRequest;
import com.sun.management.snmp.agent.SnmpMibAgent;
import com.sun.management.snmp.agent.SnmpRequestForwarder;

import com.sun.management.snmp.SnmpEngineId;
import com.sun.management.snmp.SnmpEngine;
import com.sun.management.internal.snmp.SnmpTools;

import com.sun.jdmk.trace.Trace;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.SnmpScopedPduPacket;
import com.sun.management.snmp.SnmpScopedPduRequest;
import com.sun.management.snmp.SnmpPduBulkType;
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpAckPdu;
import com.sun.management.snmp.SnmpPdu;
import com.sun.management.snmp.SnmpPduRequestType;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.SnmpVarBind;

import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.MalformedObjectNameException;

import com.sun.jdmk.ServiceName;
import com.sun.management.snmp.rfc2573.target.mib.SNMP_TARGET_MIB;
import com.sun.management.snmp.rfc2573.target.mib.SNMP_TARGET_MIBImpl;
import com.sun.management.snmp.rfc2573.v3proxy.mib.SNMP_PROXY_MIB;
import com.sun.management.snmp.rfc2573.v3proxy.mib.SNMP_PROXY_MIBImpl;
import com.sun.management.snmp.rfc2573.internal.target.SnmpJdmkTargetLcd;
import com.sun.management.snmp.rfc2573.internal.v3proxy.SnmpJdmkProxyLcd;
import com.sun.management.snmp.rfc2573.target.SnmpTargetLcd;
import com.sun.management.snmp.rfc2573.v3proxy.SnmpProxyData;
import com.sun.management.snmp.rfc2573.v3proxy.SnmpReportRuntimeException;
import com.sun.management.snmp.rfc2573.v3proxy.SnmpProxyLcd;
import com.sun.management.snmp.rfc2573.v3proxy.SnmpAgentProxy;
/**
 * Main SNMP V3 proxy class. Instances of this are added to the <CODE>SnmpV3AdaptorServer</CODE> in order to receive request to forward.
 */
public class SnmpJdmkRfc2573ProxyForwarder implements SnmpRequestForwarder {
    
    SnmpV3AdaptorServer server = null;
    MBeanServer mbserver = null;
    SnmpProxyLcd proxyLcd = null;
    SnmpTargetLcd targetLcd = null;
    String paramsFile = null;
    String targetsFile = null;
    String proxiesFile = null;
    SNMP_TARGET_MIB targetMib = null;
    SNMP_PROXY_MIB proxyMib = null;
    /**
     * Jdmk RFC 2573 proxy forwarder.
     * @param server The SNMP V3 adaptor server.
     * @param mbserver The MBean server. If null is provided, MIB MBeans will be not registered in the MBean server.
     * @param paramsFile Data file that will passed to the SnmpJdmkRfc2573LCD if no lcd are set via <CODE> setRfc2573LCD </CODE>. It contains params infos.
     * @param targetsFile Data file that will passed to the SnmpJdmkRfc2573LCD if no lcd are set via <CODE> setRfc2573LCD </CODE>. It contains targets infos.
     * @param proxiesFile Data file that will passed to the SnmpJdmkRfc2573LCD if no lcd are set via <CODE> setRfc2573LCD </CODE>. It contains proxies infos.
     */
    public SnmpJdmkRfc2573ProxyForwarder(SnmpV3AdaptorServer server,
					 MBeanServer mbserver,
					 String paramsFile,
					 String targetsFile,
					 String proxiesFile) {
	this.mbserver = mbserver;
	this.server = server;
	this.paramsFile = paramsFile;
	this.targetsFile = targetsFile;
	this.proxiesFile = proxiesFile;
    }
    /**
     * Jdmk RFC 2573 proxy forwarder.
     * @param server The SNMP V3 adaptor server.
     * @param mbserver The MBean server. If null is provided, MIB MBeans will be not registered in the MBean server.
     * @param proxyLcd The Lcd where to find proxy configuration.
     * @param targetLcd The Lcd where to find target configuration.
     */
    public SnmpJdmkRfc2573ProxyForwarder(SnmpV3AdaptorServer server,
					 MBeanServer mbserver,
					 SnmpProxyLcd proxyLcd,
					 SnmpTargetLcd targetLcd) {
	this.mbserver = mbserver;
	this.server = server;
	this.proxyLcd = proxyLcd;
	this.targetLcd = targetLcd;
    }

    /**
     * Jdmk RFC 2573 proxy forwarder.
     * @param server The SNMP V3 adaptor server.
     * @param mbserver The MBean server. If null is provided, MIB MBeans will be not registered in the MBean server.
     * @param proxiesFile The file where to find proxy configuration.
     * @param targetLcd The Lcd where to find target configuration.
     */
    public SnmpJdmkRfc2573ProxyForwarder(SnmpV3AdaptorServer server,
					 MBeanServer mbserver,
					 String proxiesFile,
					 SnmpTargetLcd targetLcd) {
	this.mbserver = mbserver;
	this.server = server;
	this.proxiesFile = proxiesFile;
	this.targetLcd = targetLcd;
    }

    /**
     * Return the passed LCD or the internally created if a file was provided.
     */
    public SnmpTargetLcd getTargetLcd() {
	return targetLcd;
    }

    /**
     * Set your our own RFC 2573 MIB target implementation.
     */
    public void setRfc2573TargetMib(SNMP_TARGET_MIB mib) {
	targetMib = mib;
    }
    
    /** Get RFC 2573 MIB target implementation. 
     */
    public SNMP_TARGET_MIB getRfc2573TargetMib() {
	return targetMib;
    }
    /**
     * Set your our own RFC 2573 MIB target implementation.
     */
    public void setRfc2573ProxyMib(SNMP_PROXY_MIB mib) {
	proxyMib = mib;
    }
    
    /** Get RFC 2573 MIB proxy implementation. 
     */
    public SNMP_PROXY_MIB getRfc2573ProxyMib() {
	return proxyMib;
    }
    
    /** 
     * Start the proxy forwarder. The configuration is read from the lcd.
     * The MIBs are registered in the adaptor.
     */
    public void start() throws Exception {
       
	if(targetLcd == null) 
	    targetLcd = new SnmpJdmkTargetLcd(server.getEngine(),
					      paramsFile,
					      targetsFile);
	targetLcd.readConfiguration();

	if(proxyLcd == null) 
	    proxyLcd = new SnmpJdmkProxyLcd(server,
					    this,
					    server.getEngine(),
					    proxiesFile,
					    targetLcd);
	proxyLcd.readConfiguration();
	
	if(targetMib == null) {
	    targetMib = new SNMP_TARGET_MIBImpl(targetLcd);
	}
	if(proxyMib == null) {
	    proxyMib = new SNMP_PROXY_MIBImpl(proxyLcd, targetLcd);
	}
	if(mbserver != null) {
	    try {
		ObjectName proxyObjName =  createProxyMibName(server, 
							      proxyMib);
		if(isTraceOn())
		    trace("init", "proxy object name : " + proxyObjName);
		ObjectName mibObjName =  createTargetMibName(server, 
							     targetMib);
		if(isTraceOn())
		    trace("init", "target object name : " + mibObjName);
		mbserver.registerMBean(targetMib, mibObjName);
		if(isTraceOn())
		    trace("init", "target mib registered");
		mbserver.registerMBean(proxyMib, proxyObjName);
		if(isTraceOn())
		    trace("init", "proxy mib registered");
	    }catch(Exception e) {
		if(isTraceOn())
		    trace("init", e.toString());
	    }
	}
	else 
	    try {
		targetMib.init();
		proxyMib.init();
	    }catch(IllegalAccessException e) {
		if(isTraceOn())
		    trace("init", e.toString());
	    }
	server.addMib(targetMib);
	server.addMib(proxyMib);
    }

    protected ObjectName createTargetMibName(SnmpV3AdaptorServer server,
					     SNMP_TARGET_MIB mib) {
	try {
	    return new ObjectName(ServiceName.DOMAIN +":"+ "mib=" + mib.getMibName());
	}catch(MalformedObjectNameException e) { return null; }
    }
    
    protected ObjectName createProxyMibName(SnmpV3AdaptorServer server,
					    SNMP_PROXY_MIB mib) {
	try {
	    return new ObjectName(ServiceName.DOMAIN +":"+ "mib=" + mib.getMibName());
	}catch(MalformedObjectNameException e) { return null; }
    }

    
    public SnmpPdu forward(SnmpPdu p) throws SnmpStatusException {
	SnmpScopedPduRequest pdu = (SnmpScopedPduRequest) p;
	int type = SnmpProxyData.READ;
	
	if(pdu.type == SnmpPdu.pduSetRequestPdu)
	    type = SnmpProxyData.WRITE;
	
	SnmpAgentProxy proxy = findProxy(pdu, type);
	if(proxy == null)
	    throw new SnmpStatusException("No proxy found for call. Do nothing.");
	SnmpMibRequest req = createMibRequest(pdu);
	try {
	    switch(pdu.type) {
	    case SnmpPdu.pduGetRequestPdu:
		proxy.get(req);
		break;
	    case SnmpPdu.pduGetNextRequestPdu:
		proxy.getNext(req);
		break;
	    case SnmpPdu.pduSetRequestPdu:
		proxy.set(req);
		break ;
	    case SnmpPdu.pduGetBulkRequestPdu:
		SnmpPduBulkType bulk = (SnmpPduBulkType) pdu;
		proxy.getBulk(req,
			      bulk.getNonRepeaters(),
			      bulk.getMaxRepetitions());
		break ;
	    }
	}catch(SnmpReportRuntimeException e) {
	    if(isDebugOn())
		debug("forward", "Received a report.");
	    return createReportPdu(pdu, e.list);
	}
	catch(SnmpStatusException e) {
	    SnmpPduRequestType resp = null;
	    resp = (SnmpPduRequestType)pdu.getResponsePdu();
	    
	    Vector v = req.getSubList();
	    int size = v.size();
	    SnmpVarBind[] list = new SnmpVarBind[size];
	    for(int i=0; i < size; i++) {
		list[i] = (SnmpVarBind) v.elementAt(i);
	    }
	    ((SnmpPdu)resp).varBindList = list;
	    resp.setErrorStatus(SnmpV3AdaptorServer.mapErrorStatus(e.getStatus(),
								   req.getVersion(),
								   pdu.type));
	    resp.setErrorIndex(e.getErrorIndex());
	    return (SnmpPdu)resp;
	}
	SnmpPdu resp = null;
	resp = pdu.getResponsePdu();
	Vector v = req.getSubList();
	int size = v.size();
	SnmpVarBind[] list = new SnmpVarBind[size];
	for(int i=0; i < size; i++) {
            list[i] = (SnmpVarBind) v.elementAt(i);
        }
	resp.varBindList = list;
	return resp;
    }
    
    private SnmpMibRequest createMibRequest(SnmpScopedPduRequest pdu) {
	SnmpVarBind[] list= pdu.varBindList;
	Vector varBind = new Vector();
        for(int i=0; i < list.length; i++) {
            varBind.addElement(list[i]);
        }
	final SnmpMibRequest result = SnmpMibAgent.newMibRequest(pdu,
								 varBind,
								 pdu.version,
								 null);
	return result;
    }
    
    private SnmpPdu createReportPdu(SnmpScopedPduRequest req, 
				    SnmpVarBindList list) {
	if(isTraceOn())
	    trace("createReportPdu", "Report to send.");
	int size = list.size();
	SnmpVarBind[] varbind = new SnmpVarBind[size];

	for(int i = 0; i < size; i++) {
	    varbind[i] = (SnmpVarBind) list.elementAt(i);
	}

	SnmpScopedPduRequest pdu = new SnmpScopedPduRequest();
	pdu.address = req.address;
	pdu.port = req.port;
	pdu.requestId = req.requestId;
	pdu.version = SnmpDefinitions.snmpVersionThree;
	pdu.msgId = req.msgId;
	pdu.type = SnmpDefinitions.pduReportPdu;
	pdu.varBindList = varbind;
	pdu.contextEngineId = req.contextEngineId;
	pdu.contextName = req.contextName;
	pdu.msgFlags = req.msgFlags;
	pdu.msgMaxSize = req.msgMaxSize;
	pdu.msgSecurityModel = req.msgSecurityModel;
	pdu.securityParameters = req.securityParameters;
	pdu.setErrorIndex(0);
	pdu.setErrorStatus(0);
	return pdu;
    }
    
    private SnmpAgentProxy findProxy(SnmpScopedPduRequest pdu,
				     int type) throws SnmpStatusException {
	SnmpAgentProxy proxy = null;
	try {
	    traceReceivedPdu(pdu);
	    int securityLevel = SnmpDefinitions.authPrivMask & (int) pdu.msgFlags;
	    proxy = proxyLcd.lookUpProxy(type,
					 pdu.type,
					 SnmpEngineId.createEngineId(pdu.contextEngineId),
					 new String(pdu.contextName),
					 SnmpDefinitions.snmpV3MsgProcessingModel,
					 pdu.msgSecurityModel,
					 pdu.securityParameters.getPrincipal(),
					 securityLevel);
	}
	catch(Exception e) {
	    throw new SnmpStatusException(e.toString());
	}
	return proxy;
    }
    
    private void traceReceivedPdu(SnmpScopedPduPacket pdu) {
	if(isDebugOn()) {
	    debug("traceReceivedPdu",  "PDU : \n context engine id : " + 
		  SnmpTools.binary2ascii(pdu.contextEngineId) + "\n context name : " +
		  new String(pdu.contextName) + "\n msg flags : " +
		  pdu.msgFlags + "\n security Model  : " +
		  pdu.msgSecurityModel + "\n principal : " + 
		  pdu.securityParameters.getPrincipal());   
	}
    }

    private boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_ADAPTOR_SNMP);
    }

    private void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_ADAPTOR_SNMP, clz, func, info);
    }

    private void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    private boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_ADAPTOR_SNMP);
    }

    private void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_ADAPTOR_SNMP, clz, func, info);
    }

    private void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    String dbgTag = "SnmpJdmkRfc2573ProxyForwarder";
}
