/*
 * @(#)file      SnmpTargetsReader.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.10
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.internal.target;

import com.sun.jdmk.trace.Trace;

import com.sun.management.snmp.rfc2573.internal.SnmpAddressFactory;
import com.sun.management.snmp.rfc2573.internal.SnmpPersistRowFileReader;
import com.sun.management.internal.snmp.SnmpPersistRowFileConsumer;
import com.sun.management.snmp.SnmpEngine;
import com.sun.management.snmp.rfc2573.target.SnmpParamsData;
import com.sun.management.snmp.rfc2573.target.SnmpTargetData;
import com.sun.management.snmp.manager.SnmpPeer;

import java.util.Enumeration;

class TargetsConsumer implements SnmpPersistRowFileConsumer {
    //Define contexts
    static final int TARGETS_NAME = 0;
    static final int DOMAIN = 1;
    static final int ADDRESS = 2;
    static final int TIMEOUT = 3;
    static final int RETRY = 4;
    static final int TAGS = 5;
    static final int PARAMS = 6;
    static final int STORAGE_TYPE = 7;

    SnmpTargetData targets = null;
    SnmpTargetsReader reader = null;
    TargetsConsumer(SnmpTargetsReader reader,
		    SnmpEngine engine,
		    SnmpPeerFactory factory) {
	targets = new SnmpTargetData(engine, factory);
	this.reader = reader;
    }
    
    void reset() {
	targets.targetName = null;
	targets.domain = null;
	targets.address = null;
	targets.timeout = -1;
	targets.retry = -1;
	targets.tags = null;
	targets.params = null;
	targets.storageType = -1;
	targets.authoritativePeer = null;
	targets.nonAuthoritativePeer = null;
    }
    
    public Object rowBegin(String row, int line) {
	reset(); 
	return targets;
    }
    
    public void rowEnd(Object rowContext, String row, int line) 
	throws Exception  {
	SnmpTargetData data = (SnmpTargetData) rowContext;
	reader.rowRead(data, row, line);
    }
    
    public void treatToken(String row,
			   int lineNumber,
			   String token,
			   int context,
			   Object rowContext,
			   boolean err) throws Exception {
	SnmpTargetData targets = (SnmpTargetData) rowContext;
	if( err || (targets == null) ) 
	    throw new Exception("Badly formed param entry. Line ["+
				lineNumber+"], data [" + row+"]");
	switch(context) {
	case TARGETS_NAME:
	    targets.targetName = token;
	    break;
	case DOMAIN:
	    targets.domain = token;
	    break;
	case ADDRESS:
	    targets.address = 
		reader.addressFactory.createAddress(targets.domain,
						    token);
	    break;
	case TIMEOUT:
	    targets.timeout = Integer.parseInt(token);
	    break;
	case RETRY:
	    targets.retry = Integer.parseInt(token);
	    break;
	case TAGS:
	    targets.tags = token;
	    break;
	case PARAMS:
	    targets.params = reader.params.getParams(token);
	    if(targets.params == null)
		throw new IllegalArgumentException("WARNING. TARGET NOT CREATED. Referenced parameter ["+ token +"] is not found. Line :" + 
						   lineNumber + 
						   ".\n" + 
						   " Row : [" + 
						   row +"]");
	    break;
	case STORAGE_TYPE:
	    targets.storageType = Integer.parseInt(token);
	default:
	}
    }
}

class SnmpTargetsReader extends SnmpPersistRowFileReader {
    SnmpParamsReader params = null;
    SnmpPeerFactory factory = null;
    SnmpAddressFactory addressFactory = null;
    SnmpEngine engine = null;
    public SnmpTargetsReader(SnmpEngine engine,
			     SnmpAddressFactory addressFactory,
			     SnmpPeerFactory factory,
			     SnmpParamsReader params,
			     String file) {
	super(file, "targetsEntry");
	this.factory = factory;
	this.addressFactory = addressFactory;
	this.params = params;
	this.engine = engine;
	init(new TargetsConsumer(this,
				 engine,
				 factory));
    }
  
    public SnmpTargetData getTarget(String name) {
	return (SnmpTargetData) getObject(name);
    }

    public Enumeration getTargets() {
	return getObjects();
    }
    
    void removeTarget(String targetName) 
	 throws IllegalArgumentException {
	 removeObject(targetName);
	 flushInFile();
    }

    public void addTarget(String targetName,
			  String domain,
			  Byte[] address,
			  int timeout,
			  int retry,
			  String tags,
			  String paramsName,
			  int storageType) throws IllegalArgumentException {
	SnmpTargetData data = new SnmpTargetData(engine,
						 factory);
	data.targetName = targetName;
	data.domain = domain;
	try {
	    data.address = addressFactory.createAddress(domain, address);
	}catch(Exception e) {
	    throw new IllegalArgumentException("Domain problem : " + e.getMessage());
	}
	data.timeout = timeout;
	data.retry = retry;
	data.tags = tags;
	data.params = params.getParams(paramsName);
	if(data.params == null) {
	    if(isDebugOn()) {
		debug("addTarget","WARNING. TARGET NOT CREATED. Referenced parameter ["+ paramsName +"] is not found.");
	    }
	    throw new IllegalArgumentException("Referenced parameter ["+ paramsName +"] is not found.");
	}
	data.storageType = storageType;
	addTarget(data);
	flushInFile();
    }

    void addTarget(SnmpTargetData data) throws  IllegalArgumentException {
	putObject(data.targetName, data);
    }

    void rowRead(SnmpTargetData d, String row, int line) throws Exception {
	SnmpPeer peer = null;
	try {
	    addTarget((SnmpTargetData) d.clone());
	}catch(Exception e) {
	    if(isDebugOn())
		debug("rowRead", e.getMessage() + " Line : "+ line + " Row :" + row);
	}
    }
    
    // TRACES & DEBUG
    //---------------
    
    boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
    }

    void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
    }

    void debug(String clz, String func, Throwable exception) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
    }

    void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    void debug(String func, Throwable exception) {
        debug(dbgTag, func, exception);
    }
    
    String dbgTag = "SnmpTargetsReader"; 
}
