/*
 * @(#)file      SnmpRfc2573UsmProxy.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.13
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.internal.v3proxy;

import com.sun.management.snmp.agent.SnmpUsmProxy;
import com.sun.management.snmp.SnmpEngine;
import com.sun.jdmk.trace.Trace;
import com.sun.management.snmp.manager.SnmpParams;
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.manager.SnmpParameters;
import com.sun.management.snmp.manager.usm.SnmpUsmParameters;
import com.sun.management.snmp.manager.SnmpRequest;
import com.sun.management.snmp.manager.usm.SnmpUsmPeer;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.SnmpUnknownModelException;


import com.sun.management.snmp.agent.SnmpMibRequest;

import com.sun.management.snmp.rfc2573.v3proxy.SnmpProxyData;
import com.sun.management.snmp.rfc2573.target.SnmpParamsData;
import com.sun.management.snmp.rfc2573.v3proxy.SnmpReportRuntimeException;

public class SnmpRfc2573UsmProxy extends SnmpUsmProxy {
    SnmpProxyData data = null;
    private String dbgTag = "SnmpRfc2573UsmProxy";
    public SnmpRfc2573UsmProxy(SnmpProxyData data, 
			       SnmpEngine engine) throws SnmpStatusException {
	super(engine, 
	      (SnmpUsmPeer) data.singleOut.nonAuthoritativePeer, 
	      null, 
	      data.proxyName);
	this.data = data;
	try {
	    init();
	}catch(Exception e) {
	    throw new SnmpStatusException(e.getMessage());
	}
	hideInvalidResponseError(false);
    }
    
    /**
     * Called when a report is received when forwarding a request. If the returned value is true, the proxy will retry the current request.
     * @param request The request containing the received report.
     * @return True means retry the call, the report has been handled. False means don't retry, the report wasn't fixed.
     * <P> If the received report is  NotInTimeWindow, the <CODE> SnmpUsmPeer </CODE> is resynchronized. If the synchronization succeeded, true is returned, false otherwise.
     */
    protected boolean handleReport(SnmpRequest request) {
	if(isDebugOn()) {
	    debug("handleReport", "Received a report");
	}

	//If the father can handle it, let it handle it.
	boolean retry = super.handleReport(request);
	if(retry) return retry;
	
	// Report to forward to the manager
	throw new SnmpReportRuntimeException(request.getReportVarBindList());
    }

    protected SnmpParams translateParameters(SnmpMibRequest req) throws SnmpStatusException {
	if(isDebugOn()) {
	    debug("translateParameters", " ");
	}
	if(isDebugOn()) {
	    debug("translateParameters", " Out params = " + data.singleOut.params);
	}
	SnmpParamsData out = data.singleOut.params;
	SnmpUsmParameters ret = null;
	SnmpUsmPeer peer = (SnmpUsmPeer) data.singleOut.nonAuthoritativePeer;
	try {
	    ret = new SnmpUsmParameters(getEngine(), out.securityName);
	}catch(SnmpUnknownModelException e) {
	    if(isDebugOn()) {
	    debug("translateParameters", " What can we do :" + e);
	    }
	}
	ret.setContextEngineId(data.contextEngineId.getBytes());
	ret.setContextName(data.contextName.getBytes());
	ret.setSecurityLevel(SnmpParamsData.translateMIBSecurityLevel(out.securityLevel)); 
	
	//Check if auth is activated for the target. 
	if((ret.getSecurityLevel() & (int) SnmpDefinitions.authNoPriv) != 0) {
	    if(peer.isTimelinessSynchronized() == false)
		try {
		    SnmpParams loc = peer.getParams();
		    peer.setParams(ret);
		    peer.processUsmTimelinessDiscovery();
		    peer.setParams(loc);
		}catch(SnmpStatusException e) {
		    if(isDebugOn()) {
			debug("translateParameters", "processUsmTimelinessDiscovery Failed. Returning null.");
		    }
		    throw new SnmpStatusException(e.getMessage());
		}
	}
	
	try {
	    ret = new SnmpUsmParameters(getEngine(), out.securityName);
	}catch(SnmpUnknownModelException e) {
	    if(isDebugOn()) {
		debug("translateParameters", " What can we do :" + e);
	    }
	    throw new SnmpStatusException(e.getMessage());
	}
	ret.setContextEngineId(data.contextEngineId.getBytes());
	ret.setContextName(data.contextName.getBytes());
	ret.setSecurityLevel(SnmpParamsData.translateMIBSecurityLevel(out.securityLevel));    

	return ret;
    }

      private boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_ADAPTOR_SNMP);
    }

    private void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_ADAPTOR_SNMP, clz, func, info);
    }

    private void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    private boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_ADAPTOR_SNMP);
    }

    private void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_ADAPTOR_SNMP, clz, func, info);
    }

    private void debug(String func, String info) {
        debug(dbgTag, func, info);
    }

}
