/*
 * @(#)file      SimpleManager2.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.8
 * @(#)lastedit  04/04/07
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// java import
//
import java.io.IOException;

// RI import
//
import com.sun.management.snmp.SnmpEventReportDispatcher;

// jdmk import
//
import com.sun.jdmk.tasks.TaskServer;
import com.sun.jdmk.tasks.DaemonTaskServer;

/**
 * The class is a simple example showing how to use the SNMP Manager API
 * and particularly how to receive an inform request.
 *
 * <p>The port number in use for listening incoming inform PDUs is 8085.
 */

public class SimpleManager2 {

    /**
     * Main entry point.
     */
    public static void main(String argv[]) {
     
        // Initialize the port number to listen for incoming inform 
	// PDUs on port 8085.
        //
        final int port = 8085;

        try {
            
	    // Create a taskServer for processing informs.
	    // This is an optional step. However using a DaemonTaskServer
	    // to process incomming PDUs makes it possible to empty
	    // the inform socket faster, thus reducing the hazards
	    // of request loss.
	    // We set the priority of the DaemonTaskServer to 
	    // Thread.NORM_PRIORITY so that emptying the socket takes 
	    // precedence over inform processing.
	    //
	    final DaemonTaskServer taskServer = new DaemonTaskServer();
	    taskServer.start(Thread.NORM_PRIORITY);
	    
            // Create a listener and dispatcher for SNMP inform requests 
	    // (SnmpEventReportDispatcher).
            // SnmpEventReportDispatcher is run as a thread and listens 
	    // for informs on the specified port.
            // Add InformListenerImpl as SnmpInformListener.
            // InformListenerImpl will receive a callback when a valid 
	    // inform PDU is received.
            //
            final SnmpEventReportDispatcher informDispatcher = 
		new SnmpEventReportDispatcher(port,null,taskServer,null);
            informDispatcher.addInformListener(new InformListenerImpl()); 
            final Thread informThread = new Thread(informDispatcher);
	    informThread.setPriority(Thread.MAX_PRIORITY);
	    informThread.start();
            println("\nNOTE: SNMP simple manager 2 initialized");
            
            // Note that you can use the same SnmpEventReportDispatcher 
	    // object for both incoming traps and informs.
            // Just add the required trap listener:
            //   informDispatcher.addTrapListener(new TrapListenerImpl()); 
            //
            
            // Here we are just going to wait for receiving inform PDUs.
            //
            println("\nNOTE: Event report listener initialized");
            println("\tand listening for incoming inform PDUs on port " + 
		    port + "...");
     
            // Here you could do whatever processing you need, for 
	    // instance you could run a graphical user interface.
            //
       
            println("\n\n>> Press <Enter> if you want to stop" + 
		    " this SNMP manager.\n");
            java.lang.System.in.read();
            
	    // Nicely stop the SnmpEventReportDispatcher.
	    //
	    informDispatcher.close();
	    taskServer.terminate();

            // That's all !
            //
            java.lang.System.exit(0);
     
        } catch (Exception e) {
            java.lang.System.err.println("SimpleManager2::main: " +
					 "Exception occurred:" + e);
            e.printStackTrace();
        }
    }

    /**
     * print/println stuff...
     **/
    private final static void println(String msg) {
	java.lang.System.out.println(msg);
    }

    private final static void print(String msg) {
	java.lang.System.out.print(msg);
    }

}
