/*
 * @(#)file      JmxMBeanAttrEntryImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.6
 * @(#)lastedit  04/04/07
 * 
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

import java.io.Serializable;
import javax.management.MBeanServer;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.agent.SnmpMib;

/**
 * The class is used for implementing the "JmxMBeanAttrEntry" group.
 * An instance of this class mirrors the value of an attribute of an 
 * MBean mirrored in the jmxMBeanTable.
 */
public class JmxMBeanAttrEntryImpl extends JmxMBeanAttrEntry {

    final private JmxMBeanEntryImpl mbeanEntry;

    /**
     * Constructor for the "JmxMBeanAttrEntry" group.
     */
    public JmxMBeanAttrEntryImpl(SnmpMib myMib, JmxMBeanEntryImpl mbeanEntry) {
	super(myMib);
	this.mbeanEntry = mbeanEntry;
    } 

    /**
     * Getter for the "JmxMBeanAttrStringValue" variable.
     */
    public String getJmxMBeanAttrStringValue() throws SnmpStatusException {
 	try {
	    final MBeanServer s = mbeanEntry.getMibServer();
	    final Object val = 
		s.getAttribute(mbeanEntry.getObjectName(),JmxMBeanAttrName);
	    return JMX_MBEAN_SERVER_MIB_Impl.
		validJmxMBeanAttrStringValueTC(String.valueOf(val));
	} catch (Exception x) {
	    return JMX_MBEAN_SERVER_MIB_Impl.
		validJmxMBeanAttrStringValueTC(String.valueOf(x));
	}
    }

    /**
     * Getter for the "JmxMBeanAttrName" variable.
     */
    public String getJmxMBeanAttrName() throws SnmpStatusException {
	return JMX_MBEAN_SERVER_MIB_Impl.
	    validJmxMBeanAttrNameTC(JmxMBeanAttrName);
    }
}
