/*
 * @(#)file      JMX_MBEAN_SERVER_MIB_Impl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.8
 * @(#)lastedit  04/04/21
 * 
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

import java.io.Serializable;
import java.util.Hashtable;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.InstanceAlreadyExistsException;
import com.sun.management.snmp.agent.SnmpMib;
import com.sun.management.snmp.agent.SnmpMibNode;
import com.sun.management.snmp.agent.SnmpMibTable;
import com.sun.management.snmp.agent.SnmpStandardObjectServer;

/**
 * The class is used for representing "JMX-MBEAN-SERVER-MIB".
 * You can edit the file if you want to modify the behavior of the MIB.
 * We override the generated JMX_MBEAN_SERVER_MIB in order to:
 * instantiate an instance of JmxMBeanServerImpl instead of JmxMBeanServer.
 */
public class JMX_MBEAN_SERVER_MIB_Impl extends JMX_MBEAN_SERVER_MIB {

    /**
     * Default constructor. Initialize the Mib tree.
     */
    public JMX_MBEAN_SERVER_MIB_Impl(MBeanServer mibServer) {
	super();
	this.mibServer = mibServer;
    }

    private final MBeanServer mibServer;
    
    /**
     * Return the MBeanServer whose content is mirrored by this MIB.
     **/
    public final MBeanServer getMibServer() {
	return mibServer;
    }

    /**
     * Factory method for "JmxMBeanServer" group MBean.
     * 
     * You can redefine this method if you need to replace the default
     * generated MBean class with your own customized class.
     * 
     * @param groupName Name of the group ("JmxMBeanServer")
     * @param groupOid  OID of this group
     * @param groupObjname ObjectName for this group (may be null)
     * @param server    MBeanServer for this group (may be null)
     * 
     * @return An instance of the MBean class generated for the
     *         "JmxMBeanServer" group (JmxMBeanServer)
     * 
     * Note that when using standard metadata,
     * the returned object must implement the "JmxMBeanServerMBean"
     * interface.
     **/
    protected synchronized Object createJmxMBeanServerMBean(String groupName,
					       String groupOid,  
					       ObjectName groupObjname, 
					       MBeanServer server)  {

        // Note that when using standard metadata,
        // the returned object must implement the "JmxMBeanServerMBean"
        // interface.
        //

	// We instantiate a JmxMBeanServerImpl object instead of the generated
	// JmxMBeanServer skeleton.
	//
	// We store the instantiated object into the mbeanServerGroup 
	// variable so that we can later access to it.
	//
        if (server != null) 
            mbeanServerGroup = new JmxMBeanServerImpl(this,server);
        else 
            mbeanServerGroup = new JmxMBeanServerImpl(this);
	return mbeanServerGroup;
    }


    /**
     * Factory method for "JmxMBeanServer" group metadata class.
     * 
     * You can redefine this method if you need to replace the default
     * generated metadata class with your own customized class.
     * 
     * @param groupName Name of the group ("JmxMBeanServer")
     * @param groupOid  OID of this group
     * @param groupObjname ObjectName for this group (may be null)
     * @param server    MBeanServer for this group (may be null)
     * 
     * @return An instance of the metadata class generated for the
     *         "JmxMBeanServer" group (JmxMBeanServerMeta)
     * 
     **/
    protected synchronized JmxMBeanServerMeta 
	createJmxMBeanServerMetaNode(String groupName,
				     String groupOid, 
				     ObjectName groupObjname, 
				     MBeanServer server)  {
	// We instantiate a JmxMBeanServerMetaImpl object instead of the 
	// generated JmxMBeanServerMeta. This will make it possible to
	// replace the generated table Meta classes by our own custom
	// implementation.
	//
	// We store the instantiated object into the mbeanServerMeta
	// variable so that we can later access to it.
	//
        mbeanServerMeta = new JmxMBeanServerMetaImpl(this, objectserver);
	return mbeanServerMeta;
    }



    /**
     * Period during which cached data will be considered valid.
     * Units are milliseconds.
     **/
    public synchronized long getCacheValidity() {
	return cacheValidity;
    }

    /**
     * Set the period during which cached data will be considered valid.
     * Units are milliseconds.
     * @param validity The new value.
     * @exception IllegalArgumentException if the provided value is < 0.
     **/
    public synchronized void setCacheValidity(long validity) {
	if (validity < 0) 
	    throw new IllegalArgumentException(String.valueOf(validity)+
					       ": value must be positive");
	cacheValidity = validity;
    }

    /**
     * Returns the registered meta for the JmxMBeanTable.
     **/
    synchronized JmxMBeanTableMetaImpl getJmxMBeanTableMeta() {
	return (JmxMBeanTableMetaImpl)getRegisteredTableMeta("JmxMBeanTable");
    }

    /** 
     * Truncate the given String if needed. This method is used to ensure
     * that the Strings returned by the MIB fit into the maximum size defined
     * by their TEXTUAL-CONVENTION. 
     **/
    private static String validString(int max, String str) {

	if(str == null) return "";

	if(str.length() > max) 
	    return str.substring(0, max);
	else return str;
    }


    /**
     * Truncate string for the DisplayString TEXTUAL-CONVENTION.
     **/
    static String validDisplayStringTC(String str) {
	return validString(DISPLAY_STRING_MAX_LENGTH,str);
    }

    /**
     * Truncate string for the JmxMBeanObjectNameTC TEXTUAL-CONVENTION.
     **/
    static String validJmxMBeanObjectNameTC(String str) {
	return validString(MBEAN_OBJECT_NAME_MAX_LENGTH,str);
    }

    /**
     * Truncate string for the JmxJavaClassNameTC TEXTUAL-CONVENTION.
     **/
    static String validJmxJavaClassNameTC(String str) {
	return validString(JAVA_CLASS_NAME_MAX_LENGTH,str);
    }

    /**
     * Truncate string for the JmxMBeanAttrNameTC TEXTUAL-CONVENTION.
     **/
    static String validJmxMBeanAttrNameTC(String str) {
	return validString(MBEAN_ATTR_NAME_MAX_LENGTH,str);
    }

    /**
     * Truncate string for the JmxMBeanAttrStringValueTC TEXTUAL-CONVENTION.
     **/
    static String validJmxMBeanAttrStringValueTC(String str) {
	return validString(MBEAN_ATTR_VALUE_MAX_LENGTH,str);
    }

    /**
     * Maximum size for DisplayString
     **/
    static final int DISPLAY_STRING_MAX_LENGTH    = 255;

    /**
     * Maximum size for JmxMBeanObjectNameTC
     **/
    static final int MBEAN_OBJECT_NAME_MAX_LENGTH = 1023;

    /**
     * Maximum size for JmxJavaClassNameTC
     **/
    static final int JAVA_CLASS_NAME_MAX_LENGTH   = 1023;

    /**
     * Maximum size for JmxMBeanAttrNameTC
     **/
    static final int MBEAN_ATTR_NAME_MAX_LENGTH   = 1023;

    /**
     * Maximum size for JmxMBeanAttrStringValueTC
     **/
    static final int MBEAN_ATTR_VALUE_MAX_LENGTH  = 1023;

    /**
     * Period during which cached data will be considered valid.
     * Units are milliseconds.
     **/
    static final long MIB_CACHE_VALIDITY = 1000;

    long cacheValidity = MIB_CACHE_VALIDITY;

    JmxMBeanServerImpl     mbeanServerGroup = null;
    JmxMBeanServerMetaImpl mbeanServerMeta  = null;
}
