/*
 * @(#)file      JmxMBeanServerImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.7
 * @(#)lastedit  04/04/07
 * 
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// java imports
//
import java.io.Serializable;
import java.util.Set;
import java.util.Map;
import java.util.List;
import java.util.HashMap;
import java.util.Iterator;

// jmx/jdmk imports
//
import javax.management.MBeanServer;
import com.sun.management.snmp.SnmpString;
import com.sun.management.snmp.SnmpInt;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpStatusException;
import javax.management.ObjectName;
import javax.management.MBeanServerNotification;
import javax.management.MBeanServerConnection;
import javax.management.Notification;
import javax.management.MBeanNotificationInfo;
import javax.management.NotificationEmitter;
import javax.management.NotificationListener;
import javax.management.NotificationFilter;
import javax.management.NotificationBroadcasterSupport;
import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.MalformedObjectNameException;
import com.sun.management.snmp.agent.SnmpMib;
import com.sun.management.snmp.agent.SnmpMibTable;
import com.sun.management.snmp.agent.SnmpMibSubRequest;
import com.sun.management.snmp.EnumRowStatus;


/**
 * This class is used for instrumenting the "JmxMBeanServer" group.
 * It does not extend the mibgen generated JmxMBeanServer skeleton, but
 * instead simply implements the generated JmxMBeanServerMBean interface.
 * Since we called mibgen with the -X:no-table-access and since the
 * tables have been implemented as virtual tables, this class no longer
 * need any reference to the generated TableJmxMBeanTable and 
 * TableJmxMBeanAttrTable classes, which are no longer used.
 */
public class JmxMBeanServerImpl implements JmxMBeanServerMBean {
    
    // The MIB to which this object belongs.
    //
    final JMX_MBEAN_SERVER_MIB_Impl myMib;

    /**
     * Constructor for the "JmxMBeanServer" group.
     */
    public JmxMBeanServerImpl(SnmpMib myMib) {
	this.myMib = (JMX_MBEAN_SERVER_MIB_Impl)myMib;
    }


    /**
     * Constructor for the "JmxMBeanServer" group.
     * 
     */
    public JmxMBeanServerImpl(SnmpMib myMib, MBeanServer server) {
	// We do not want to have the mibgen generated MBeans
	// registered in the MBeanServer: this would cause
	// an infinite loop, since the registered MBean would be
	// automatically mirrored in the MIB, hence triggering the
	// registration of a new generated entry MBean etc....
	// 
	// We thus ignore the passed MBeanServer.
	//
	// Note: we could still register the generated MBeans in an
	//       MBeanServer ptovided that it is a different MBeanServer
	//       that returned by getMibServer(). 
	//
	this.myMib = (JMX_MBEAN_SERVER_MIB_Impl)myMib;
    }


    /**
     * The MBeanServer that is mirrored by this MIB. 
     * Note: the MIB is not expected to be registered in an MBeanServer.
     *       It is only registered in the SnmpAdaptorServer.
     **/
    public MBeanServer getMibServer() {
	return  myMib.getMibServer();
    }


    /**
     * Getter for the "JmxMBeanCount" variable.
     */
    public Integer getJmxMBeanCount() throws SnmpStatusException {
	try {
	    return new Integer(myMib.getJmxMBeanTableMeta().getTableSize());
	} catch (Exception x) {
            throw new SnmpStatusException(SnmpStatusException.snmpRspGenErr);
	}
    }

}
