/*
 * @(#)file      AsyncRspHandler.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.7
 * @(#)lastedit  04/04/07
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// JDMK import
//
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.manager.SnmpRequest;
import com.sun.management.snmp.manager.SnmpRequestHandler;

/**
 * This class is a simple implementation of response handler.
 * The class is used by the AsyncManager manager for handling responses.
 */
public class AsyncRspHandler implements SnmpRequestHandler {
   
    /**
     * Create a handler.
     */
    public AsyncRspHandler() {
    }
   
    public void processSnmpPollData(SnmpRequest request, int errStatus, 
				    int errIndex, SnmpVarBindList vblist) {
     
        println("AsyncRspHandler::processSnmpPollData: " + 
		request.toString());
        println("errStatus = " + SnmpRequest.snmpErrorToString(errStatus) +
		" errIndex = " + errIndex);
     
        // Check if a result is available.
        //
        if (request.getRequestStatus() == SnmpRequest.stResultsAvailable) {
            // A result is available. Go and get it !
            //
            SnmpVarBindList result = request.getResponseVarBindList();
            println("Result = " + result.varBindListToString());
        }
    }
   
    public void processSnmpPollTimeout(SnmpRequest request) {
        
        println("AsyncRspHandler::processSnmpPollTimeout: " + 
		request.toString());
        
        if (request.getRequestStatus() == SnmpRequest.stResultsAvailable) {
            // A result is available. Go and get it !
            //
            SnmpVarBindList result = request.getResponseVarBindList();
            println("Result = " + result.varBindListToString());
        }
    }
      
    public void processSnmpInternalError(SnmpRequest request, 
					 String errmsg) {
        
        println("AsyncRspHandler::processSnmpInternalError: " + 
		request.toString());
        println("Error is: " + errmsg);
    }

    /**
     * print/println stuff...
     */

    private final static void println(String msg) {
	java.lang.System.out.println(msg);
    }

    private final static void print(String msg) {
	java.lang.System.out.print(msg);
    }
}
