/*
 * @(#)file      TableDemoTableImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.7
 * @(#)lastedit  04/04/21
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// java imports
//
import java.io.Serializable;
import java.util.Vector;

// jmx imports
//
import javax.management.MBeanServer;
import javax.management.ObjectName;
import com.sun.management.snmp.SnmpCounter;
import com.sun.management.snmp.SnmpCounter64;
import com.sun.management.snmp.SnmpGauge;
import com.sun.management.snmp.SnmpInt;
import com.sun.management.snmp.SnmpUnsignedInt;
import com.sun.management.snmp.SnmpIpAddress;
import com.sun.management.snmp.SnmpTimeticks;
import com.sun.management.snmp.SnmpOpaque;
import com.sun.management.snmp.SnmpString;
import com.sun.management.snmp.SnmpStringFixed;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpNull;
import com.sun.management.snmp.SnmpValue;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpStatusException;

// jdmk imports
//
import com.sun.management.snmp.agent.SnmpIndex;
import com.sun.management.snmp.agent.SnmpMib;
import com.sun.management.snmp.agent.SnmpMibTable;
import com.sun.management.snmp.agent.SnmpMibSubRequest;
import com.sun.management.snmp.agent.SnmpTableEntryFactory;
import com.sun.management.snmp.agent.SnmpTableCallbackHandler;
import com.sun.management.snmp.agent.SnmpTableSupport;

/**
 * This class is used to customize the "DemoTable" table.
 * It shows how to update some variable in the group containing
 * this table whenever an entry is added or removed.
 * It also shows how to instantiate a customized class for entries
 * by overriding the default generated entry factory method.
 * 
 */
public class TableDemoTableImpl extends TableDemoTable {

    /**
     * Constructor for the table. We override this constructor in order
     * to keep a reference on the group MBean that contains this table.
     */
    public TableDemoTableImpl(SnmpMib myMib, DemoImpl myGroup) {
        super(myMib);

	// Keep a reference on the containing group.
	// 
	this.myGroup = myGroup;
    }

    /**
     * Constructor for the table. We override this constructor in order
     * to keep a reference on the group MBean that contains this table.
     */
    public TableDemoTableImpl(SnmpMib myMib, MBeanServer server, 
			      DemoImpl myGroup) {
        super(myMib,server);

	// Keep a reference on the containing group.
	// 
	this.myGroup = myGroup;
    }


    // ------------------------------------------------------------
    // 
    // Redefines the "removeEntryCb" method defined in "SnmpTableSupport".
    // See the "SnmpTableSupport" Javadoc API for more details.
    // 
    // We redefine this method in order to call myGroup.removeRowCb()
    // which will update the value of the demoInteger variable.
    // ------------------------------------------------------------

    public void removeEntryCb(int pos, SnmpOid row, ObjectName name,
                Object entry, SnmpMibTable meta)
            throws SnmpStatusException {
	super.removeEntryCb(pos,row,name,entry,meta);

	// Call removeRowCB() on the containing group in order to
	// update the demoInteger variable.
	//
	myGroup.removeRowCb();
    }

    // ------------------------------------------------------------
    // 
    // Redefines the "addEntryCb" method defined in "SnmpTableSupport".
    // See the "SnmpTableSupport" Javadoc API for more details.
    // 
    // We redefine this method in order to call myGroup.addRowCb()
    // which will update the value of the demoInteger variable.
    // ------------------------------------------------------------

    public void addEntryCb(int pos, SnmpOid row, ObjectName name, 
			   Object entry, SnmpMibTable meta) 
	throws SnmpStatusException {
	super.addEntryCb(pos,row,name,entry,meta);

	// Call addRowCB() on the containing group in order to
	// update the demoInteger variable.
	//
	myGroup.addRowCb();
    }

    /**
     * Factory method for "DemoEntry" entry MBean class.
     * 
     * We redefine this method in order to replace the default
     * generated MBean class for entries with our own customized class.
     * Note that we do nothing special in the customized class:
     * We show how to override this method simply for the sake of the 
     * example.
     *
     * @return An instance of the customized MBean class that implements 
     *         the "DemoEntry" conceptual row.
     * 
     */

    public Object createDemoEntryMBean(SnmpMibSubRequest req,
                SnmpOid rowOid, int depth, ObjectName entryObjName,
                SnmpMibTable meta, Integer  aDemoTableIndex)
            throws SnmpStatusException  {

        DemoEntryImpl entry = new DemoEntryImpl(theMib);
        entry.DemoTableIndex = aDemoTableIndex;
        return entry;
    }


    /**
     * Reference to the group that contains this table.
     */
    private DemoImpl myGroup;

}
